@@ -2,15 +2,14 @@ package Inline::Wrapper;
 #
 #   Inline::* module dynamic loader and useful wrapper routines
 #
-#   infi/08
+#   $Id: Wrapper.pm 8 2008-12-27 19:16:54Z infidel $
 #
 #   POD documentation after __END__
 #
-#   TODO: write 'unload()'
 
 use strict;
 use warnings;
-use Carp qw( croak );
+use Carp qw( carp croak );
 use Data::Dumper;
 use vars qw( $TRUE $FALSE $VERSION );
 BEGIN { $INC{'Inline::Wrapper.pm'} ||= __FILE__ };  # recursive use check
@@ -20,7 +19,7 @@ use Inline::Wrapper::Module;                        # individual code modules
 ### VARS
 ###
 
-$VERSION = '0.02';
+$VERSION = '0.03';
 *TRUE    = \1;
 *FALSE   = \0;
 
@@ -42,7 +41,7 @@ my $PARAMS = {
     language    => sub {
                      defined( $_[0] ) and exists( $LANGS->{$_[0]} )
                          ? $_[0]
-                         : ( warn sprintf( "Invalid language: %s; using %s",
+                         : ( carp sprintf( "Invalid language: %s; using %s",
                                            $_[0], $DEFAULTS->{language} )
                                and $DEFAULTS->{language} )
                    },
@@ -88,7 +87,6 @@ sub initialize
 ###
 
 # Load a code module named $modname from $base_dir with $lang_extension
-# XXX: Check for dupes, return the pre-built object if it's a dupe
 sub load
 {
     my( $self, $modname, @args ) = @_;
@@ -119,6 +117,7 @@ sub load
     # Create a new module object
     my $module = Inline::Wrapper::Module->new(
             module_name     => $modname,
+            lang_ext        => $self->_lang_ext(),
             $self->_settings(),
             %args,
     );
@@ -130,6 +129,17 @@ sub load
     return( @functions );
 }
 
+# Completely unload a loaded $modname, rendering its functions uncallable
+sub unload
+{
+    my( $self, $modname ) = @_;
+
+    carp "$modname not loaded" and return
+        unless( ref( $self->_module( $modname ) ) );
+
+    return( $self->_del_module( $modname ) && $modname );
+}
+
 # Run a $modname::$funcname function, passing it @args
 sub run
 {
@@ -155,7 +165,7 @@ sub functions
     my( $self, $modname ) = @_;
 
     my $module = $self->_module( $modname );
-    warn "Module '$modname' not loaded"
+    carp "Module '$modname' not loaded"
         and return()
             unless( ref( $module ) );
 
@@ -244,10 +254,10 @@ sub add_language
 {
     my( $self, $language, $lang_ext ) = @_;
 
-    warn "add_language(): Language not set; read the docs"
+    carp "add_language(): Language not set; read the docs"
         and return
             unless( $language );
-    warn "add_language(): Language extension not set; read the docs"
+    carp "add_language(): Language extension not set; read the docs"
         and return
             unless( $lang_ext );
 
@@ -279,9 +289,6 @@ sub _module
 {
     my( $self, $modname ) = @_;
 
-#    croak "Module '$modname' not loaded"
-#        unless( exists( $self->{modules}->{$modname} ) );
-
     return( $self->{modules}->{$modname} );
 }
 
@@ -292,6 +299,14 @@ sub _add_module
     return( $self->{modules}->{$modname} = $module );
 }
 
+sub _del_module
+{
+    my( $self, $modname ) = @_;
+
+    # Namespace is deleted by $module->DESTROY()
+    return( delete( $self->{modules}->{$modname} ) );
+}
+
 ###
 ### PRIVATE UTILITY ROUTINES
 ###
@@ -326,137 +341,139 @@ sample.pl:
     base_dir    => '.',
  );
 
- my @symbols = $inline->load( 'sample_module' );
+ my @symbols = $inline->load( 'answer' );
 
- my @retvals = $inline->run( 'sample_module', 'sample_func', 2, 3 );
+ my @retvals = $inline->run( 'answer', 'the_answer', 3, 56 );
 
- print $retvals[0], "\n";
+ print "The answer is: ", $retvals[0], "\n";
 
  exit(0);
 
-sample_module.c:
+answer.c:
 
- int sample_func( int arg1, int arg2 ) {
-     return arg1 * arg2;
+ int the_answer( int arg1, int arg2 ) {
+     return ( arg1 * arg2 ) >> 2;
  }
 
 =head1 DESCRIPTION
 
-Inline::Wrapper provides wrapper routines to make embedding another
-language into a Perl application more convenient.
+B<Inline::Wrapper> provides wrapper routines around L<Inline> to make
+embedding functions from another language into a Perl application much
+more convenient.
 
-Instead of having to include the external code in after __END__ in Perl
-source code, you can have separate, individually configurable language module
-directories to contain all of your code.
+Instead of having to include the external code in a Perl source file after
+the B<__END__> directive, B<Inline::Wrapper> allows you to have separate,
+individually-configurable module repositories to more easily manage all
+of your external application code.
 
 =head1 FEATURES
 
-Inline::Wrapper provides the following:
+B<Inline::Wrapper> provides the following features:
 
 =over 4
 
-=item * Support for all languages that Inline.pm supports.
+=item * Support for all languages supported by L<Inline>.
 
-=item * A single, unified interface to running loaded module functions.
+=item * A single, unified interface for running loaded module functions.
 
 =item * Loading of files containing pure source code, only in their
-respective language.
+respective language, so you can outsource maintenance of these modules to
+another party.
 
-=item * Individually configurable module directories.
+=item * Individually-configurable module directories.
 
 =item * Automatic, run-time module reloading upon file modification time
 detection.
 
-=item * No more namespace pollution.
+=item * No more namespace pollution.  All module symbols are loaded into
+their own individual, private namespaces, so they won't collide with your
+code or each other.
 
 =back
 
 =head1 CONSTRUCTOR
 
-=over 4
+=head2 new()
 
-=item B<$obj = new(> I<[ var =E<gt> value, ... ]> B<)>
+    my $wrapper = Inline::Wrapper->new(
+          language        => 'C',
+          base_dir        => 'src/code/C',
+          auto_reload     => 1,
+    );
 
 Create a new B<Inline::Wrapper> object, with the appropriate attributes (if
 specified).
 
-RETURNS: blessed $object, or undef on failure.
+B<ARGUMENTS>:
 
-ARGUMENTS:
-
-All arguments are of the hash form  Var => Value.  B<new()> will complain
+All arguments are of the hash form  Var => Value.  L</"new()"> will complain
 and croak if they do not follow this form.
 
-B<NOTE:> The arguments to B<new()> become the defaults used by B<load()>.
-You can individually configure loaded modules using B<load()>, as well.
+B<NOTE:> The arguments to L</"new()"> become the defaults used by
+L</"load()">.  You can individually configure loaded modules using
+L</"load()">, as well.
 
 =over 4
 
-=item B<language>           [ default: B<Lua> ]
+=item I<language>           [ default: B<Lua> ]
 
-Set to the DEFAULT language for which you wish to load modules, if not
-explicitly specified via B<load()>.
+Set to the default language for which you wish to load modules, if not
+explicitly specified via L</"load()">.
 
 B<NOTE>: It defaults to Lua because that is what I wrote this module for.
 Just pass in the argument if you don't like that.
 
 B<ALSO NOTE:> Currently only a couple of "known" languages are hard-coded
 into this module.  If you wish to use others, don't pass this argument, and
-use the B<add_language()> method, documented below, after the object has
-been instantiated.
+use the L</"add_language()"> method after the object has been instantiated.
 
-=item B<auto_reload>        [ default: B<FALSE> ]
+=item I<auto_reload>        [ default: B<FALSE> ]
 
 Set to a TRUE value to default to automatically checking if modules have
-been changed since the last B<load()>, and reload them if necessary.
+been changed since the last L</"load()">, and reload them if necessary.
 
-=item B<base_dir>           [ default: B<'.'> ]
+=item I<base_dir>           [ default: B<'.'> ]
 
 Set to the default base directory from which you wish to load all modules.
 
-=back
+B<RETURNS>: blessed $object, or undef on failure.
 
 =back
 
-=head2 Example
-
- my $wrapper = Inline::Wrapper->new(
-        language        => 'C',
-        base_dir        => 'src/code/C',
-        auto_reload     => 1,
- );
-
 =head1 METHODS
 
-=over 4
+=head2 initialize()
 
-=item B<initialize( )>
+    $obj->initialize();
 
-Initialize arguments.  If you are subclassing, overload this, not B<new()>.
+Initialize arguments.  If you are subclassing, overload this, not L</"new()">.
 
-Returns nothing.
+Generally only called from within L</"new()">.
 
-=item B<load(> I<$modname [, %arguments ]> B<)>
+=head2 load()
 
-The workhorse.  Loads the actual module itself, importing its symbols into a
-private namespace, and making them available to call via B<run()>.
+    my @functions = $obj->load( $modname, %args );
 
-$modname is REQUIRED.  It corresponds to the base filename, without
-extension, loaded from the B<base_dir>.  See the
-L<"Details of steps taken by load()"> section, Step 3, a few lines down
-for clarification of how pathname resolution is done.  It is also how you
-will refer to this particular module from your program.
+The workhorse.  Loads the actual module referred to by I<$modname>,
+imports its symbols into a private namespace, and makes them available to
+call via L</"run()">.
 
-B<This method accepts all of the same arguments as new().>  Thus, you can
-set the DEFAULTS via B<new()>, yet still individually configure module
-components different from the defaults, if desired.
+B<ARGUMENTS:>
+
+I<$modname> is REQUIRED.  It corresponds to the base filename, without
+extension, loaded from the I<base_dir>.  See the
+L</"Details of steps taken by load()"> section, Step 3, for clarification
+of how pathname resolution is done.  I<$modname> is also how you will refer
+to this particular module from your program, so keep track of it.
+
+This method accepts all of the same arguments as L</"new()">.  Thus, you can
+set the defaults via L</"new()">, yet still individually configure module
+components differently from the defaults, if desired.
 
 Returns a list of @functions made available by loading $modname, or warns
 and returns an empty list if unsuccessful.
 
-=back
-
-=head2 Details of steps taken by load()
+=head3 Details of steps taken by load()
 
 Since this is the real guts of this module, here are the exact steps taken
 when loading the module, doing pathname resolution, etc.
@@ -467,21 +484,16 @@ when loading the module, doing pathname resolution, etc.
 if so, returns the list of available functions in that module immediately.
 
 =item 2. Creates a new L<Inline::Wrapper::Module> container object with any
-supplied %arguments, or the defaults you specified with
-B<Inline::Wrapper->new()>.
+supplied %arguments, or the defaults you specified with L</"new()">.
 
 =item 3. Constructs a path to the specified $modname, as follows:
 
-=over 2
-
-=item C<$base_dir + $path_separator + $modname + $lang_ext>
-
-=back
+    $base_dir + $path_separator + $modname + $lang_ext
 
 =over 4
 
-=item I<$base_dir> is taken either from the default created with B<new()>, or
-the explicitly supplied base_dir argument to B<load()>.
+=item I<$base_dir> is taken either from the default created with
+L</"new()">, or the explicitly supplied base_dir argument to L</"load()">.
 
 =item I<$path_separator> is just the appropriate path separator for your OS.
 
@@ -491,7 +503,7 @@ can supply your own subdirectories, as well.  'foo' is just as valid as
 
 =item I<$lang_ext> is taken from a data structure that defaults to
 common-sense filename extensions on a per-language basis.  Any of these can
-be overridden via the B<add_language()> method.
+be overridden via the L</"add_language()"> method.
 
 =back
 
@@ -508,12 +520,24 @@ of loaded, available functions provided by the module.
 
 =back
 
-=over 4
+=head2 unload()
 
-=item B<run(> I<$modname, $function [, @args ]> B<)>
+    $obj->unload( $modname );
 
-Run the named $function that you loaded from $modname, with the specified
-@arguments (if any).
+Completely unload the module identified by I<$modname>, and render its
+functions uncallable.
+
+This will actually go through and destroy L<Inline::Wrapper::Module> object,
+as well as the code module's corresponding private namespace.
+
+Returns I<$modname> (TRUE) upon success, carps and returns undef on failure.
+
+=head2 run()
+
+    my @retvals = $obj->run( $modname, $function, @arguments );
+
+Run the named I<$function> that you loaded from I<$modname>, with the
+specified I<@arguments> (if any).
 
 Assuming a successful compilation (you are checking for errors, right?),
 this will execute the function provided by the loaded module.  Call syntax
@@ -521,83 +545,93 @@ and everything is up to the function provided.  This simply executes the sub
 that L<Inline> loaded as-is, but in its own private namespace to keep your
 app clean.
 
-Returns @list of actual return values provided by the module itself.
-Whatever the module returns in its native language, you get back.
+Returns I<@retvals>, consisting of the actual return values provided by
+the module function itself.  Whatever the function returns in its native
+language, that's what you get.
 
-=item B<modules( )>
+=head2 modules()
 
-Returns @list of available, loaded and ready module names, or the empty list
-if no modules have been loaded.
+    my @modules = $obj->modules();
 
-=item B<functions(> I<$modname> B<)>
+Returns a list of loaded module names, or the empty list if no modules
+have been (successfully) loaded.
 
-Returns @list of functions provided by $modname, or the empty list.
+=head2 functions()
 
-=back
+    my @functions = $obj->functions( $modname );
+
+Returns a list of loaded I<@functions>, which were made available by loading
+I<$modname>.
 
 =head1 ACCESSORS
 
 Various accessors that allow you to inspect or change the default settings
 after creating the object.
 
-=over 4
+=head2 base_dir()
 
-=item B<base_dir( )>
+    my $base_dir = $obj->base_dir();
 
-Returns the base_dir attribute from the object.
+Returns the default I<base_dir> attribute from the object.
 
-=item B<set_base_dir(> I<$path> B<)>
+=head2 set_base_dir()
 
-Sets the base_dir attribute of the object, and returns whatever it ended
-up being set to.
+    $obj->set_base_dir( '/some/path' );
 
-B<NOTE:> Only affects modules loaded after this setting was made.
+Sets the default I<base_dir> attribute of the object, and returns whatever
+it ended up being set to.
 
-=item B<auto_reload( )>
+B<NOTE:> Only affects modules loaded I<after> this setting was made.
 
-Returns a $boolean as to whether or not the current DEFAULT auto_reload()
-setting is enabled.
+=head2 auto_reload()
 
-=item B<set_auto_reload(> I<$boolean> B<)>
+    my $bool = $obj->auto_reload();
 
-Sets the auto_reload attribute of the object, and returns whatever it ended
-up being set to.
+Returns a $boolean as to whether or not the current default I<auto_reload>
+setting is enabled for new modules.
 
-B<NOTE:> Only affects modules loaded after this setting was made.
+=head2 set_auto_reload()
 
-=item B<language( )>
+    $obj->set_auto_reload( 1 );
 
-Returns the language attribute of the object.
+Sets the default I<auto_reload> attribute of the object, and returns
+whatever it ended up being set to.
 
-=item B<set_language(> I<$lang> B<)>
+B<NOTE:> Only affects modules loaded I<after> this setting was made.
 
-Sets the language attribute of the object, and returns whatever it ended up
-being set to.
+=head2 language()
 
-B<NOTE:> Only affects modules loaded after this setting was made.
+    my $lang = $obj->language();
+
+Returns the default I<language> attribute of the object.
+
+=head2 set_language()
+
+    $obj->set_language( 'C' );
+
+Sets the default I<language> attribute of the object, and returns whatever
+it ended up being set to.
+
+B<NOTE:> Only affects modules loaded I<after> this setting was made.
 
 B<ALSO NOTE:> This checks for "valid" languages via a pretty naive method.
 Currently only a couple are hard-coded.  However, you can add your own
-languages via the B<add_language()> method, described next.
+languages via the L</"add_language()"> method.
 
-=item B<add_language(> I<$language => $extension> B<)>
+=head2 add_language()
+
+    $obj->add_language( 'Lojban' => '.xkcd' );
 
 Adds a language to the "known languages" table, allowing you to later use
-set_language( $lang ).
+L</"set_language()">.
 
-REQUIRES a $language name (e.g. 'Python') and a filename $extension (e.g.
-'.py'), which will be used in pathname resolution, as described under
-B<load()>.
+This can also be used to set a new extension to an existing language.
 
-Returns TRUE if successful, issues warnings and returns FALSE otherwise.
+REQUIRES a I<$language> name (e.g. 'Python') and a filename I<$extension>
+(e.g. '.py'), which will be used in pathname resolution, as described under
+L</"load()">.
 
-B<NOTE:>If you I<CHANGE> a language filename extension with a module
-already loaded with B<auto_reload> enabled, and don't rename the underlying
-file, it'll probably freak during dynamic pathname construction, thinking
-the file has been removed.  I'll fix this in a later version.  For now,
-Don't Do That(tm).
-
-=back
+Returns TRUE if successful, carps and returns FALSE otherwise.
 
 =head1 SEE ALSO
 
@@ -605,6 +639,8 @@ L<Inline::Wrapper::Module>
 
 The L<Inline> documentation.
 
+The L<Inline::FAQ> list.
+
 The examples/ directory of this module's distribution.
 
 =head1 ACKNOWLEDGEMENTS
@@ -619,6 +655,9 @@ Please kindly read through this documentation and the B<examples/>
 thoroughly, before emailing me with questions.  Your answer is likely
 in here.
 
+Also make sure that your issue is actually with B<Inline::Wrapper> and not
+with L<Inline> itself.
+
 Jason McManus (INFIDEL) -- infidel@cpan.org
 
 =head1 LICENSE
